--mray.ex
--Newtonian Telescope Spot Diagrams
--for Euphoria v1.5+
--Kenneth Roger
--emwa68b@prodigy.com

include mouse.e
include graphics.e

constant
  gmreq = 18,  --graph mode requested
  vpw = 570,   --viewport width
  vph = 390,   --viewport height
  vpx0 = (640-vpw)/2,  --viewport x-origin
  vpy0 = (480-vph)/2,  --viewport y-origin
  grx0 = vpx0+96,
  fgcolor = BRIGHT_GREEN,  --foreground color
  pi = 3.141592654,
  pixpermm = 600.0,  --(spot scale) pixels per mm
  arcsecperrad = 60*60*180/pi,  --arc seconds per radian
  airyarcsecxap = 279.32,  --Airy disc dia.(arc sec) x aperature(mm),
  airymmperfr = airyarcsecxap/arcsecperrad  --Airy disc dia.(mm) / fratio
constant
  apers = { 3, 4, 6, 8, 10, 12, 14, 16, 18, 20 }, napers=length(apers),
  frats = { 4, 5, 6, 8, 10, 12, 15 }, nfrats=length(frats)

atom
  aper,  --aperature (mirror diameter)
  frat,  --f/ratio (focal length / aperature)
  quit,  --quit if non-zero, else continue
  yp,  --y position of a projected ray at the focal plane
  zp,   --z position of a projected ray at the focal plane
  iaper, ifrat
sequence
  y0,  --y positions of the spot diagram rays at the aperature
  z0,  --z positions of the spot diagram rays at the aperature
  xp,  --x positions of the focal plane (in and out of focus)
  zref  --z positions of the central ray at the focal plane
  y0=repeat(0.0,225)
  z0=repeat(0.0,225)
  xp=repeat(0.0,5)
  zref=repeat(0.0,5)

procedure border()
  draw_line(fgcolor,
    {{vpx0,vpy0},
    {vpw-1+vpx0,vpy0},
    {vpw-1+vpx0,vph-1+vpy0},
    {vpx0,vph-1+vpy0},
    {vpx0,vpy0}})
  draw_line(fgcolor,{{grx0,vpy0},{grx0,vph-1+vpy0}})
  end procedure

procedure makemenu()
  atom kq, ka, kf, kx
  text_color(fgcolor)
  clear_screen()
  border()
  kq=3+floor(vpy0/16)
  ka=kq+2
  kf=ka+napers+2
  kx=2+floor(vpx0/8)
  position(kq,kx) puts(1,"[ ] exit")
  position(ka,kx) puts(1,"Aperature")
  for i=1 to napers do
    position(ka+i,kx)
    if i=iaper then printf(1,"[x] %d in.",apers[i])
    else printf(1,"[ ] %d in.",apers[i]) end if
    end for
  position(kf,kx) puts(1,"F/Ratio")
  for i=1 to nfrats do
    position(kf+i,kx)
    if i=ifrat then printf(1,"[x] %d in.",frats[i])
    else printf(1,"[ ] %d in.",frats[i]) end if
    end for
  aper=apers[iaper]*25.4
  frat=frats[ifrat]
  quit=0
  end procedure

procedure querymenu()
  object r
  atom kq, ka, kf, ke, mxp, myp, my
  mouse_events(LEFT_UP)
  mouse_pointer(1) --show
  kq=2
  ka=kq+2
  kf=ka+napers+2
  ke=kf+nfrats
  while 1 do
    r=get_mouse()
    if sequence(r) then
      mxp=r[2]
      myp=r[3]
      if mxp>vpx0 and mxp<grx0 and myp>vpy0 and myp<(vpy0+vph) then
    my=1+floor((myp-vpy0)/16)
    if my<ka then quit=1 exit
    elsif my>ka and my<(kf-1) then iaper=my-ka exit
    elsif my>kf and my<=ke then ifrat=my-kf exit end if
    end if
      end if
    end while
  mouse_pointer(0) --hide
  end procedure

procedure outtextxy(integer x, integer y, sequence s)
  position(1+(y+vpy0)/16,1+(x+grx0)/8)
  puts(1,s)
  end procedure

procedure putpixel(integer x, integer y)
  if x>0 and x<vpw and y>0 and y<vph then
    pixel(fgcolor,{x+grx0,y+vpy0}) end if
  end procedure

procedure circle(integer x, integer y, integer r)
  if r<1 then r=1 end if
  ellipse(fgcolor,0,{x-r+grx0,y-r+vpy0},{x+r+grx0,y+r+vpy0})
  end procedure

procedure aperspot()
  --sets the spot diagram y0[] and z0[] for unit aperature
  atom rs0, theta, ring
  for isp=2 to 225 do
    if isp<=7 then ring=2.0 theta=(7.0-isp)/6.0
    elsif isp<=19 then ring=3.0 theta=(19.0-isp)/12.0
    elsif isp<=37 then ring=4.0 theta=(37.0-isp)/18.0
    elsif isp<=61 then ring=5.0 theta=(61.0-isp)/24.0
    elsif isp<=93 then ring=6.0 theta=(93.0-isp)/32.0
    elsif isp<=131 then ring=7.0 theta=(131.0-isp)/38.0
    elsif isp<=175 then ring=8.0 theta=(175.0-isp)/44.0
    else ring=9.0 theta=(225.0-isp)/50.0 end if
    rs0=ring/18.0
    theta=theta*2.0*pi
    y0[isp]=rs0*cos(theta)
    z0[isp]=rs0*sin(theta)
    end for
  end procedure

procedure project(integer isp, integer ip, atom lm0, atom mu0, atom nu0,
  atom r0)
  --projects the isp'th ray to the focal plane at xp[ip]
  --lm0, mu0, nu0 are direction cosines of the incoming ray
  --r0 is the radius of curvature of the mirror
  atom r2yz, x0, lmn, mun, nun, aa, lmp, mup, nup, yzro, zzro
  yzro=aper*y0[isp]  --y of this ray on the mirror
  zzro=aper*z0[isp]  --z of this ray on the mirror
  r2yz=yzro*yzro+zzro*zzro
  x0=r2yz/(2.0*r0)   --x of this ray on the mirror
  lmn=r0/sqrt(r0*r0+r2yz)  --three direction cosines of the mirror normal
  mun=yzro*lmn/r0
  nun=zzro*lmn/r0
  aa=2.0*(lmn*lm0+mun*mu0+nun*nu0)
  lmp=aa*lmn-lm0  --three direction cosines of the reflected ray
  mup=aa*mun-mu0
  nup=aa*nun-nu0
  yp=yzro-mup*(xp[ip]-x0)/lmp  --y of this ray at the focal plane (q.e.d.)
  zp=zzro-nup*(xp[ip]-x0)/lmp  --z of this ray at the focal plane (q.e.d.)
  end procedure

procedure dolabels()
  sequence str1, str2
  integer radius1, radius2
  text_color(fgcolor)
  outtextxy(115,16,"Newtonian Telescope Spot Diagram")
  str1=sprintf("aperature %.1fmm",aper)
  outtextxy(30,336,str1)
  str1=sprintf("focal ratio f/%.1f",frat)
  outtextxy(30,352,str1)
  outtextxy(30,368,"parabolic mirror")
  str1=sprintf("view = %.2f deg/10mm",572.96/(frat*aper))
  outtextxy(30,384,str1)
  radius1=floor(0.01*pixpermm)
  circle(222,332,radius1)
  outtextxy(240,336,"photo film grain (fine)")
  str1=sprintf("  dia. = 0.020mm, %.1f\" arc",(0.02*arcsecperrad/(frat*aper)))
  outtextxy(240,352,str1)
  radius2=floor(airymmperfr*frat*pixpermm/2.0)
  circle(222,364,radius2)
  outtextxy(240,368,"diffraction disc")
  str1=sprintf("  dia. = %.3fmm,",(airymmperfr*frat))
  str2=sprintf(" %.1f\" arc",(airyarcsecxap/aper))
  outtextxy(240,384,str1&str2)
  outtextxy(410,64,"10.0mm")
  outtextxy(410,80,"   off")
  outtextxy(410,96,"  axis")
  outtextxy(410,144," 2.0mm")
  outtextxy(410,160,"   off")
  outtextxy(410,176,"  axis")
  outtextxy(410,224," 0.0mm")
  outtextxy(410,240,"   off")
  outtextxy(410,256,"  axis")
  outtextxy(40,288,"0.4mm")
  outtextxy(40,304," near")
  outtextxy(118,288,"0.2mm")
  outtextxy(118,304," near")
  outtextxy(196,288,"0.0mm")
  outtextxy(274,288,"0.2mm")
  outtextxy(274,304,"  far")
  outtextxy(352,288,"0.4mm")
  outtextxy(352,304,"  far")
  outtextxy(410,288," focus")
  outtextxy(410,304," error")
  end procedure

procedure dospots()
  atom alpha, alpha1, lm0, mu0, nu0, r0, fpx, fpdx, x0
  integer xpixel, ypixel
  r0= -2.0*aper*frat
  fpx= -1.0*aper*frat
  fpdx= 0.2
  x0= -1.0
  for ip=1 to 5 do xp[ip]=fpx+(3-ip)*fpdx end for  --focal plane x-positions
  alpha1= 0.01  --guess at alpha for 1 mm off-axis
  yp=0.0
  zp=0.0
  for ia=1 to 3 do  --off-axis loop
    if ia=1 then alpha=0.0
    elsif ia=2 then alpha=2.0*alpha1
    else alpha=10.0*alpha1 end if
    lm0=cos(alpha)
    mu0=0.0
    nu0=sin(alpha)
    if ia=2 then
      project(1,1,lm0,mu0,nu0,r0)  --trial projection of central ray
      alpha1=alpha1/zp  --corrected alpha1
      alpha=2.0*alpha1
      lm0=cos(alpha)
      nu0=sin(alpha)
      end if
    for ip=1 to 5 do  --determine central ray z-positions
      project(1,1,lm0,mu0,nu0,r0)
      zref[ip]=zp
      end for
    for isp=20 to 225 do  --spot loop, assume rays 1-19 are blocked
    for ip=1 to 5 do  --projection plane loop (in and out of focus)
      project(isp,ip,lm0,mu0,nu0,r0)
      xpixel=80*ip+floor(yp*pixpermm)-20
      ypixel=316-80*ia-floor((zp-zref[ip])*pixpermm)
      putpixel(xpixel,ypixel)  --plot the ray (q.e.d.)
      end for
      end for
    end for
  end procedure

--main
  integer gmret
  gmret=graphics_mode(gmreq)
  if gmret=0 then
    aperspot()
    iaper=4
    ifrat=2
    quit=0
    while not quit do
      makemenu()
      dolabels()
      dospots()
      querymenu()
      end while
    gmret=graphics_mode(3)
  else puts(1,"VGA is required\n") end if
--done
